#include "basicga.h"
#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <wait.h>
#include <fcntl.h>

void BasicGa::run(){
    numEvals = 0;
    initParams();
    params->evaluator = this;
    loadParameters();
    load();
    buildGenome();
    buildPopulation();
    for( int i = 0; i < params->numGens; i ++ ){
        if( params->showEvery == -1 )
            showEval = true;
        pop->runGeneration();
        if( params->showEvery != 0 )
            if( i%params->showEvery == 0 || i == params->numGens -1 )
                showBest();
    }
    pop->logGenStats();
}

void BasicGa::initParams(){
    params = new Params;
}

void BasicGa::loadParameters(){
    params->popSize = 10;
    params->numGens = 10;
    params->mutationProb = .01;
    params->crossoverProb = .5;
    params->elitists = 0;

    FILE* file = fopen("parameters", "r");
    if( file <= 0 ) return;
    char line[1024];
    float value;
    int i;
    while( fgets( line, 1024, file ) ){
        if( line[strlen(line)-1] == '\n' )
            line[strlen(line)-1] = '\0';
        parseParam( line );
    }
    printf("%s\n", params->toString() );
    fclose( file );
}

void BasicGa::parseParam( char* line ){
    char param[1024];
    char s[1024];
    float value;
    if( sscanf( line, "%s %f", &param, &value ) == 2 ){
             if( strcmp( param, "popsize" )         == 0) params->popSize           = (int) value;
        else if( strcmp( param, "numgens" )         == 0) params->numGens           = (int) value;
        else if( strcmp( param, "mutationprob" )    == 0) params->mutationProb      = value;
        else if( strcmp( param, "crossoverprob" )   == 0) params->crossoverProb     = value;
        else if( strcmp( param, "elitists" )        == 0) params->elitists          = (int) value;
        else if( strcmp( param, "showevery"  )      == 0) params->showEvery         = (int)value;
        else if( strcmp( param, "logevery"  )       == 0) params->logEvery          = (int)value;
        else if( strcmp( param, "avgmappedfitness") == 0) params->avgMappedFitness  = value;
        else if( strcmp( param, "maxmappedfitness") == 0) params->maxMappedFitness  = value;
        else if( strcmp( param, "memetic")          == 0) params->memetic           = (value != 0);
        else if( strcmp( param, "randomseeds")      == 0) params->randomSeeds       = (int) value;
        else if( strcmp( param, "quality")          == 0) params->quality           = value;
    }
    if( sscanf( line, "%s %s", &param, &s ) ){
        if( strcmp( param, "selection")        == 0) {
            if( strcmp(s, "roulette") ==0 ){
                params->selectionType = SELECTION_ROULETTE;
                printf("using roulette wheel selection\n");
            }
            if( strcmp(s, "chc")==0 ){
                params->selectionType = SELECTION_CHC;
                printf("using chc selection\n");
            }
        }
    }
}

void BasicGa::load(){
}
void BasicGa::buildGenome(){
}
void BasicGa::buildPopulation(){
    pop = new Population();
    pop->setParams( params );
    pop->initialize();
}
float BasicGa::eval( Genome* g ){
    return 1.0f;
}
float BasicGa::evaluate( Genome* g ){
    numEvals++;
    float fitness = eval( g );
    if( params->quality != 0.0f && fitness >= params->quality ){
        FILE* file = fopen("logs/quality", "a" );
        printf("achieved %f after %i\n", fitness, numEvals );
        fprintf(file, "%i\n", numEvals );
        fclose( file );
        exit(0);
    }
    return fitness;
}
void BasicGa::showBest(){
    showEval = true;
    printf("best: %s\n", pop->genome(0)->toString().Get() );
    eval( pop->genome(0) );
    showEval = false;
}
void BasicGa::show( char* bitString ){
    printf("BasicGa::show( %s )\n", bitString );

    initParams();
    params->evaluator = this;
    loadParameters();
    load();
    buildGenome();
    Genome g;
    g.setParams( params );
    g.initialize();
    g.fromString( bitString );
    showEval = true;
    eval( &g );
}

