#include <math.h>
#include "chrom.h"
Chrom::Chrom():
    length(0),
    offset(0)
{ }
void Chrom::setType( ChromType type ){
    this->type = type;
    if( type == BIT )
        length = 1;
    if( type == INT ){
        length = 8;
        intmin = 0;
        intmax = 255;
    }
    if( type == FLOAT ){
        length = 8;
        floatmin = 0.0f;
        floatmax = 0.0f;
    }
}

void Chrom::setMin( int i ){
    assert( type == INT );
    intmin = i;
    updateLength();
}

void Chrom::setMax( int i ){
    assert( type == INT );
    intmax = i;
    updateLength();
}

void Chrom::setMin( float f ){
    assert( type == FLOAT );
    floatmin = f;
    updateLength();
}

void Chrom::setMax( float f ){
    assert( type == FLOAT );
    floatmax = f;
    updateLength();
}

void Chrom::updateLength(){
    int range;
    switch( type ){
        case( BIT ): 
            length = 1;
            break;
        case( INT ):
            range = intmax - intmin + 1;
            length = (int) ceil( log( range ) / log(2) );
            //printf("range: %i, bits : %i, lg: %f\n", range, length, log2(range) );
            break;
        case( FLOAT ):
            length = 8;
            break;
    }
}

bool Chrom::getB( Genome* g ){
    assert( type == BIT );
    return g->bits[offset];
}

int Chrom::getI( Genome* g ){
    assert( type == INT );
    int answer = 0;
    int power = 1;
    //printf("parsing : ");
    //for( int bit = offset; bit < offset + length; bit ++ )
      //  printf("%i", g->bits[bit] );
    for( int bit = offset + length-1; bit >= offset; bit-- ){
        answer += g->bits[bit] * power;
        power *= 2;
    }
    answer = (( answer - intmin) % ( intmax - intmin + 1) ) + intmin;
    //printf("into : %i\n", answer );
    assert( answer >= intmin );
    assert( answer <= intmax );
    return answer;
}

float Chrom::getF( Genome* g ){
    assert( type == FLOAT );
    float fanswer = 0;
    int ianswer = 0;
    int power = 1;
    for( int bit = offset + length-1; bit >= offset; bit-- ){
        ianswer += g->bits[bit]* power;
        power *= 2;
    }
    fanswer = (((float)ianswer)/256.0f)*(floatmax-floatmin)+floatmin;
    return fanswer;
}
    
char* Chrom::toString(){
    switch( type ){
        case( BIT ):
            sprintf( string, "BIT @ %i", offset );
            break;
        case( INT ):
            sprintf( string, "INT %i->%i (%i) @ %i", intmin, intmax, length, offset );
            break;
        case( FLOAT ):
            sprintf( string, "FLOAT %f->%f (%i) @ %i", floatmin, floatmax, length, offset );
            break;
    }
    return string;
}
