#include "population.h"

void Population::runGeneration(){
    switch( params->selectionType ){
    case SELECTION_ROULETTE:
        eval();
        mapFitnesses();
        sortPopulation();
        if( generation%params->logEvery == 0 )
            logData();
        selection();
        generation++;
        z = generation%2;
        crossover();
        mutation();
        if( params->memetic )
            memetic();
    break;
    case SELECTION_CHC:
        //eval parents
        eval();
        if( generation%params->logEvery == 0 )
            logData();
        //make kids
        randDuplicate();
        generation++;
        z = generation%2;
        //modify kids
        crossover();
        mutation();
        if( params->memetic )
            memetic();
        //evaluate
        eval();
        //sort them all
        sortBothPopulations();
        //keep the top half
        keepBest();
    break;
    }
}

void Population::mapFitnesses(){
    mappedFitnessStats.clear();
    for( int i = 0; i < genomes[z].Size(); i ++ ){
        mapFitness( genomes[z][i] );
        mappedFitnessStats.add( genomes[z][i]->mappedFitness, i );
        genomes[z][i]->lastSlot = mappedFitnessStats.sum;
    }
}

Population::Population(){
    generation = 0;
    z = 0;
}
void Population::setParams( Params* params ){
    this->params = params;
}
void Population::initialize(){
    Genome* g[2];
    assert( params->popSize > 0 );
    assert( params->genomeLength > 0 );
    for( int i = 0; i < params->popSize; i ++ ){
        g[0] = new Genome;
        g[1] = new Genome;
        g[0]->setParams( params );
        g[0]->initialize();
        g[0]->randomize();
        *g[1] = *g[0];
        genomes[0]          .Append( g[0] );
        sortedGenomes[0]    .Append( g[0] );
        bothPopulations     .Append( g[0] );
        
        genomes[1]          .Append( g[1] );
        sortedGenomes[1]    .Append( g[1] );
        bothPopulations     .Append( g[1] );

    }
}

nArray<nString> Population::toStrings(){
    //string[0] = '\0';
    nArray<nString> answer;
    for( int i = 0; i < params->popSize; i ++ ){
        answer.Append( (sortedGenomes[z][i]->toString() + "\n") );
        //sprintf( string, "%s%s\n", string, genomes[z][i].toString().Get() );
    }
    //return string;
    return answer;
}
void Population::eval(){
    assert( params );
    assert( params->evaluator );
    fitnessStats.clear();
    for( int i = 0; i < params->popSize; i ++ ){
        genomes[z][i]->eval();
        fitnessStats.add( genomes[z][i]->fitness, i );
    }
}
int compar( const void* a, const void* b ){
    Genome* x = *((Genome**)a);
    Genome* y = *((Genome**)b);
    //printf("comparing : %s < %s\n", x->toString().Get(), y->toString().Get() );
    return *x < *y;
    //return *(*((Genome**)a)) < *(*((Genome**)b));
}
void Population::sortPopulation(){
    //printf("genomes[0]: %s\n", genomes[z][0]->toString().Get() );
    qsort( &sortedGenomes[z][0], params->popSize, sizeof( Genome* ), compar );
}
void Population::sortBothPopulations(){
    qsort( &bothPopulations[0], bothPopulations.Size(), sizeof( Genome* ), compar );
}


void Population::mapFitness( Genome* g ){
    if( fitnessStats.range == 0.0f ) 
        g->mappedFitness = 1.0f;
    else {
        if( g->fitness < fitnessStats.avg ){
            float r = (g->fitness - fitnessStats.min ) / fitnessStats.minrange;
            g->mappedFitness = r * ( params->avgMappedFitness-1.0f) + 1.0f;
        }
        else {
            float r = (g->fitness - fitnessStats.avg ) / fitnessStats.maxrange;
            g->mappedFitness = r * ( params->maxMappedFitness-params->avgMappedFitness) + params->avgMappedFitness;
        }
    }
}
void Population::selection(){
    int i = 0;
    while( i < params->elitists ){
        *genomes[1-z][i] = *sortedGenomes[z][i];
        i++;
    }
    while( i < params->elitists + params->randomSeeds ){
        genomes[1-z][i]->randomize();
        i++;
    }
    while( i < genomes[z].Size() ){
        float choice = frand( mappedFitnessStats.sum );
        int chosen = -1;
        int low = 0;
        int high = params->popSize - 1;
        int mid = 0;
        int mid0 = -1;
        while( mid!= mid0 ){
            mid0 = mid;
            mid = (high + low)/2;
            if( genomes[z][mid]->lastSlot > choice )
                if( mid == 0 || genomes[z][mid-1]->lastSlot < choice ){
                    chosen = mid;
                    break;
                }
                else
                    high = mid;
            else
                low = mid;
        }
        if( chosen == -1 )
            chosen = params->popSize-1;
        *genomes[1-z][i] = *genomes[z][chosen];
        i++;
    }
}
void Population::randDuplicate(){
    bool* taken;
    taken = new bool[genomes[z].Size()];
    for( int i = 0; i < genomes[z].Size(); i ++ )
        taken[i] = false;
    for( int i = 0; i < genomes[z].Size(); i ++ ){
        int j;
        do j = irand( 0, genomes[z].Size()-1 );
        while ( taken[j] );
        taken[j] = true;
        *genomes[1-z][j] = *genomes[z][i];
    }
}
void Population::keepBest(){
    nArray<Genome> bestGenomes ( genomes[z].Size(), 10 );
    for( int i = 0; i < genomes[z].Size(); i ++ ){
        bestGenomes.Append( *bothPopulations[i] );
    }
    for( int i = 0; i < genomes[z].Size(); i ++ ){
        *genomes[z][i] = bestGenomes[i];
    }
}

void Population::crossover(){
    for( int i = params->elitists; i < genomes[z].Size()-1; i +=2 ){
        genomes[z][i]->crossoverWith( genomes[z][i+1] );
    }
}

void Population::mutation(){
    for( int i = params->elitists; i < genomes[z].Size(); i ++ ){
        genomes[z][i]->mutate();
    }
}
void Population::memetic(){
    for( int i = params->elitists; i < genomes[z].Size(); i ++ ){
        genomes[z][i]->memetic();
    }
}

void Population::logData(){
    FILE* file = fopen("logs/genlogs", "a" );
    fprintf( file, "generation %i done\n", generation );
    fprintf( file, "fitnessstats : %s\n", fitnessStats.toString().Get() );
    fprintf( file, "mappedfitnessstats : %s\n", mappedFitnessStats.toString().Get() );
    nArray<nString> strings = toStrings();
    for( int i = 0; i < strings.Size(); i ++ )
        fprintf(file, "%s", strings[i].Get() );
    fclose( file );
    
    printf("generation %i done\n", generation );
    printf("fitnessstats : %s\n", fitnessStats.toString().Get() );
    printf("mappedfitnessstats : %s\n", mappedFitnessStats.toString().Get() );
    for( int i = 0; i < strings.Size(); i ++ )
        printf("%s", strings[i].Get() );
    genFitnessStats.add( fitnessStats );
}

void Population::logGenStats(){
    FILE* file;
    nArray<nString> strings;
    
    file = fopen("logs/avglogs", "a" );
    strings = genFitnessStats.avg();
    for( int i =0; i < strings.Size(); i ++ )
        fprintf( file, "%s", strings[i].Get() );
    fclose( file );
    
    file = fopen("logs/minlogs", "a" );
    strings = genFitnessStats.min();
    for( int i =0; i < strings.Size(); i ++ )
        fprintf( file, "%s", strings[i].Get() );
    fclose( file );
    
    file = fopen("logs/maxlogs", "a" );
    strings = genFitnessStats.max();
    for( int i =0; i < strings.Size(); i ++ )
        fprintf( file, "%s", strings[i].Get() );
    fclose( file );
}

