#include "ga.h"
#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <wait.h>
#include <fcntl.h>
void Ga::buildGenome(){
    repres.setParams( params );
    Chrom c;

    c.setType( c.INT );
    c.setMin( 0 );
    c.setMax( targets.Size()-1 );

    char name[1024];
    for( int i = 0; i < assets.Size(); i ++ ){
        //strncpy( name, "", 1024 );
        sprintf( name, "%s:%s%i", assets[i].platform.Get(), assets[i].type.Get(), i );
        //printf("name: %s\n", name );
        repres.add( c, name );
        //printf("name: %s\n", name );
    }

    //printf("making a bittree of size : %i\n", params->genomeLength );
    params->bitTree = new BitTree( params->genomeLength );
    //printf("done making a bittree of size : %i\n", params->genomeLength );
}

float Ga::eval( Genome* g ){
    if( showEval )
        printf("evaluating %s\n", g->toString().Get() );
    repres.startParse( g );
    for( int i = 0; i < assets.Size(); i ++ ){
        int alloc = repres.parseI( (RepresHandle) i );
        assets[i].target = targets[alloc].name;
        //printf("assigned %s\n", assets[i].toString().Get());
    }
    repres.stopParse();
    float f = runPlan();
    if( showEval )
        printf("fit : %f\n", f );
    return f;
}

float Ga::runPlan( ){
    Platform p;
    Target t;
    //clear plans for all platforms
    for( int i = 0; i < platforms.Size(); i ++ ){
        platforms[i].wps.Clear();
    }
    //create plans
    for( int i = 0; i < assets.Size(); i ++ ){
        p.name = assets[i].platform;
        Platform* platform = platforms.Find( p );
        t.name = assets[i].target;
        Target* target = targets.Find(t);
        wp w;
        w.lat = target->lat;
        w.lng = target->lng;
        //printf("adding wp for : %s @ %f,%f\n", assets[i].toString().Get(), w.lat, w.lng );
        platform->wps.Append( w );
    }
    if( showEval )
        for( int i = 0; i < platforms.Size(); i ++ ){
            printf("%s\n", platforms[i].toString().Get() );
        }
    saveRoutes();
    
    char* args[9];
    args[0] = "/cs/miles/nebula/bin/linuxd/go";
    args[1] = "sim";
    args[2] = "timescale";
    if( showEval ){
        args[3] = ".5";
        args[4] = "";
        args[5] = "";
    }
    else{
        args[3] = "50";
        args[4] = "nographics";
        args[5] = "simpleai";
    }
    args[6] = "import";
    args[7] = new char[1024];
    sprintf( args[7], "%s/%s", "/cs/miles/ga/sim", params->mission );
    args[8] = 0;
    pid_t kid;
    if(( kid = fork() ) == 0 ){
        if( showEval ){
            printf("execing ");
            for( int i = 0; i < 9; i ++ )
                printf("%s ", args[i] );
            printf("\n");
        }
        int fdout = open( "/dev/null", O_WRONLY, 0666 );
        assert( fdout >= 0 );
        dup2( fdout, 1 );
        close( fdout );
        execvp( args[0], args );
    }
    else if( kid < 0 ){
        printf("error %i occured\n", kid );
    }
        waitpid( kid, NULL, 0 );
    
    return parseFitness();
}

void Ga::saveRoutes(){
    char prefixs[1024];
    sprintf( prefixs, "%s/route-", params->mission );
    nString prefix = prefixs;
    for( int i = 0; i < platforms.Size(); i ++ ){
        nString fileName = prefix + platforms[i].name;
        //printf("save route %s\n", fileName.Get() );
        FILE* file = fopen( fileName.Get(), "w" );
        if( file == 0 ){
            printf("opening %s to save route too failed\n", fileName.Get() );
            continue;
        }
        float time = 0.0f;
        float lat = params->carrierlat;
        float lng = params->carrierlng;
        float alt = 0.0f;
        fprintf( file, "%0.1f %.4f %.4f ALT %0.1f TAKEOFF \n", time, lat, lng, alt );
        for( int j = 0; j < platforms[i].wps.Size(); j ++ ){
            time = 0.0f;
            lat = platforms[i].wps[j].lat;
            lng = platforms[i].wps[j].lng;
            alt = 0.0f;
            assert( j < platforms[i].assets.Size() );
            Asset* asset = platforms[i].assets[j];
            //printf("type: %s\n", asset->type.Get() );
            //printf("target: %s\n", asset->target.Get() );
            fprintf( file, "%0.1f %.4f %.4f ALT %0.1f FIRE %s %s\n", time, lat, lng, alt, asset->type.Get(), asset->target.Get() );
        }
        lat = params->carrierlat;
        lng = params->carrierlng;
        fprintf( file, "%0.1f %.4f %.4f ALT %0.1f LAND\n", time, lat, lng, alt );
        fclose( file );
    }

}

float Ga::parseFitness(){
    FILE* file = fopen( "/cs/miles/nebula/data/sim/messagelog", "r" );
    if( file == 0 ){
        printf("unable to open messagelog to parse fitness\n" );
        return 0.0f;
    }
    char line[1024];

    char objectName[1024];
    float value;
    float survivalProb;
    float redDamage = 0;
    float blueDamage = 0;
    float fitness = 0;
    enum { 
        BLUE,
        RED
    };
    int mode = BLUE;
    while( fgets( line, 1024, file ) ){
        if( line[strlen(line)-1]=='\n' )line[strlen(line)-1] = '\0';
        if( strcmp( line, "RED" ) == 0 )
            mode = RED;
        else if ( sscanf( line, "\t%s $%f: %f\n", &objectName, &value, &survivalProb ) == 3 ){
            if( showEval )
                printf("%-16s $%.2f at %.2f\n", objectName, value, survivalProb );  
            if( mode == BLUE )
                blueDamage += value * ( 1 - survivalProb );
            else if( mode == RED )
                redDamage += value * ( 1 - survivalProb );
        }
    }
    fclose( file );
    fitness = blueDamage - redDamage;
    if( showEval ){
        printf("red total damage : %f\n", blueDamage );
        printf("blue total damage : %f\n", redDamage );
        printf("fitness : %f\n", fitness );
    }
    return fitness;
}
