#ifndef GA_H
#define GA_H
#include <math.h>
#include <stdio.h>
#include "basicga.h"
#include "simparams.h"

class wp
{
    public:
    float lat, lng;
    wp(){}
};
class Asset
{
    public:
        nString type;
        nString platform;
        nString target;
    Asset(){}
    Asset( const char* a, const char* b):
        type( a ),
        platform( b )
        {}
    nString toString(){
        char answer[1024];
        if( target == "" )
            sprintf( answer, "%10s on %10s", type.Get(), platform.Get() );
        else
            sprintf( answer, "%10s on %10s targeting %10s", type.Get(), platform.Get(), target.Get() );
        return nString( answer );
    }
    void operator =( const Asset& rhs ){
        type = rhs.type;
        platform = rhs.platform;
        target = rhs.target;
    }
};
class Platform
{
    public:
        nString name;
        nString type;
        float value;
        nArray<wp> wps;
        nArray<Asset*> assets;
        Platform(){}
    Platform( char* name, char* type, float value ){
        this->name = name;
        this->type = type;
        this->value = value;
    }
    nString toString(){
        char answer[1024];
        sprintf( answer, "%10s %10s $%.2f", name.Get(), type.Get(), value );
        for( int i = 0; i < wps.Size(); i ++ )
            sprintf( answer, "%s\nwp%i %f %f", answer, i, wps[i].lat, wps[i].lng );
        return nString( answer );
    }
    bool operator == ( Platform rhs ){
        return name == rhs.name;
    }
};
class Target
{
    public:
        nString name;
        float value;
        float lat, lng;

    Target(){}
    Target( const char* name, float value, float lat, float lng ){
        this->name = name;
        this->value = value;
        this->lat = lat;
        this->lng = lng;
    }
    nString toString(){
        char answer[1024];
        sprintf( answer, "%12s $%.2f @%.2f,%.2f", name.Get(), value, lat, lng );
        return nString( answer );
    }
    bool operator == ( Target rhs ){
        return name == rhs.name;
    }
};
        
class Ga : public BasicGa
{
protected:
    nArray<Platform> platforms;
    nArray<Asset> assets;
    nArray<Target> targets;
    SimParams* params;
public:
    Ga():BasicGa(){}
protected:
    virtual void initParams();
    virtual void parseParam( char* line );
    virtual void load();
        void loadPlatforms();
        void loadTargets();
        void loadThreats();
    virtual void buildGenome();
    virtual float eval( Genome* rhs );
        float runPlan();
            void saveRoutes();
            float parseFitness();
};
#endif
