#include "ga.h"
#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <wait.h>
#include <fcntl.h>
void Ga::initParams(){
    BasicGa::params = params = new SimParams;
}
    
void Ga::parseParam( char* line ){
    BasicGa::parseParam( line );
    char param[1024];
    if( sscanf( line, "%s", &param ) == 1 ){
        if(      strcmp( param, "carrierlat" )  == 0) assert( sscanf( line, "%s %f", &param, &params->carrierlat ) == 2 );
        else if( strcmp( param, "carrierlng" )  == 0) assert( sscanf( line, "%s %f", &param, &params->carrierlng ) == 2 );
        else if( strcmp( param, "mission" )     == 0) assert( sscanf( line, "%s %s", &param, &params->mission    ) == 2 );
    }
}

void Ga::load(){
    loadPlatforms();
    loadTargets();
    loadThreats();
}

void Ga::loadPlatforms(){
    char filename[1024];
    strncpy( filename, "", 1024 );
    sprintf( filename, "%s/platforms", params->mission );
    FILE* file = fopen( filename, "r" );
    if( file == 0 ){
        printf("opening %s failed\n", filename );
        return;
    }
    char* line = 0;
    size_t len = 0;
    char name[1024];
    strncpy( name, "", 1024 );
    char type[1024];
    strncpy( type, "", 1024 );
    float value;
    float lat, lng;
    char assets[10][1024];
    for( int i = 0; i < 10; i ++ )
        strncpy( assets[i], "", 1024 );
    int elements;
    while( getline( &line, &len, file ) != -1 ) {
        elements = sscanf( line, "%s %s %f %f %f %s %s %s %s %s %s %s %s %s %s\n", &name, &type, &value, &lat, &lng, &assets[0], &assets[1], &assets[2], &assets[3], &assets[4], &assets[5], &assets[6], &assets[7], &assets[8], &assets[9] );
       // sscanf( line, "%s %s %s %s\n", &name, &type, &value, &assets );
        printf("PLATFORM %30s %32s $%.2f @ %.2f,%.2f\n", name, type, value, lat, lng );
        Platform p(name, type, value) ;
        for( int i = 0; i < elements - 5; i ++ )
            if( strcmp( assets[i], "NONE" ) != 0 ){
                printf("ASSET %33s\n", assets[i] );
                Asset a( assets[i], p.name.Get() );
                this->assets.Append( a );
                p.assets.Append( &a );
                //p.assets.Append( &this->assets.Back() );
                //printf("p.assets.[j] : %s\n", p.assets.Back()->toString().Get() );
            }
        platforms.Append( p );
    }
    fclose( file) ;
}
void Ga::loadTargets(){
    char filename[1024];
    sprintf( filename, "%s/targetlist", params->mission );
    FILE* file = fopen( filename, "r" );
    if( file == 0 ){
        printf("opening %s failed\n", filename);
        return;
    }
    char name[1024];
    char type[1024];
    float value;
    float lat;
    float lng;
    while( int i = fscanf( file, "%s %s %f %f %f", &name, &type, &value, &lat, &lng ) == 5  ){
        printf("TARGET %32s %32s $%.2f @ %.2f,%.2f\n", name, type, value, lat, lng ); 
        Target t( name, value, lat, lng );
        targets.Append( t );
    }
    fclose( file );
}
void Ga::loadThreats(){
    char filename[1024];
    sprintf( filename, "%s/threatlist", params->mission );
    FILE* file = fopen( filename, "r" );
    if( file == 0 ){
        printf("opening %s failed\n", filename );
        return;
    }
    char name[1024];
    char type[1024];
    float value;
    float lat;
    float lng;
    float radius;
    float lethality;
    while( int i = fscanf( file, "%s %s %f %f %f %f %f", &name, &type, &value, &lat, &lng, &radius, &lethality) == 7 ){
        printf("THREAT %32s %32s $%.2f @ %.2f,%.2f -> %.2f (%.2f)\n", name, type, value, lat, lng, radius, lethality );
    }
    fclose( file );
}
