#ifndef ROUTER_H
#define ROUTER_H

#include <math.h>
#include <stdlib.h>
#include "path.h"
#include "point.h"
#include "enkiduo/heap.h"

class Router
{
    class Node{
    public:
        point pos;
        float costTo;
        float costFrom;
        Node* parent;
        float cost;
        Node( point _pos, float _costTo, float _costFrom, Node* _parent ):
            pos( _pos),
            costTo(_costTo),
            costFrom(_costFrom),
            parent(_parent)
        {
            cost = costTo + costFrom;
        }
    };

    int w;
    float** map;
    public:
    Router( int w )
    {
        this->w = w;
        map = new float*[w];
        for( int i = 0; i < w; i ++ ){
            map[i] = new float[w];
            for( int j = 0; j < w; j ++ )
                map[i][j] = 0;
        }
    }
    void coot(){
        for( int i = 0; i < w; i ++ ){
            for( int j = 0; j < w; j ++ )
                n_printf("%f ", map[i][j] );
            n_printf("\n");
        }
    }
    void randomize(){
        for( int i = 0; i < w; i ++ )
            for( int j = 0; j < w; j ++ )
                map[i][j] = rand()%10;
    }
    float estimate( point a, point b )
    {
        return sqrt( pow( a.x-b.x,2) + pow(a.y-b.y,2) );
    }
    float traverseCost( point a, point b )
    {
        return sqrt( pow( a.x-b.x,2) + pow (a.y-b.y,2) ) * map[a.x][a.y];
    }
    float getMapVal( int x, int y )
    {
        return map[x][y];
    }
    void setMapVal( int x, int y, int v )
    {
        map[x][y] = v;
    }
    Path findRoute( point startPoint, point goalPoint ) 
    {
        Path path;
        Heap<Node*> open( 2048 );
        Path closed;
        float** best;
        best = new float*[w];
        for( int i = 0; i < w; i ++ ){
            best[i] = new float[w];
            for( int j = 0; j < w; j ++ )
                best[i][j] = 10000000;
        }

        best[startPoint.x][startPoint.y] = 0;
        Node start( startPoint, 0, estimate( startPoint, goalPoint) , 0 );
        open.add( start.cost , &start );
        Node* current;
        while( open.length() > 0 ){
            float i;
            open.pop( i, current );
            if( current->costTo <= best[current->pos.x][current->pos.y]) {
                //printf("popped %i %i with cost %f\n", current->pos.x, current->pos.y, current->cost );

                if( current->pos == goalPoint )
                {
                  //  printf("dest found in %f %f\n", current->pos.x, current->pos.y );
                    for( Node* tmp = current;
                            tmp;
                            tmp = tmp->parent ){
                        printf("route : %i %i\n", tmp->pos.x, tmp->pos.y );
                        path.addFront( tmp->pos );
                    }
                    return path;
                }
    
                for( int i = -1; i <= 1; i ++ )
                    for( int j = -1; j <= 1; j ++ )
                        if( i != 0 || j != 0 ){
                            point p( current->pos.x+i, current->pos.y+j );
                            if( p.x >= 0 && p.y >= 0 && p.x < w && p.y < w ){
                                Node* newNode = new Node( p,
                                                        current->cost + traverseCost( current->pos, p ),
                                                        estimate( p, goalPoint ),
                                                        current );
                                if( newNode->costTo < best[p.x][p.y] ){
                                    best[p.x][p.y] = newNode->costTo;
                                    open.add( newNode->cost, newNode );
                                }
                            }
                        }
                closed.add( current->pos );
            }
//            else
               // printf("%f %f ( %f) failed to beat %f\n", current->pos.x, current->pos.y, current->cost, best[current->pos.x][current->pos.y] );
        }
        printf("warning: no path found\n");
    }
};
#endif
        
