#include "tspgenome.h"
#include "graph.h"
#include "tspparams.h"
void TspGenome::randomize(){
    static bool* taken = new bool[params->genomeLength];
    for( int i = 0; i < params->genomeLength; i ++ )
        taken[i] = false;
    for( int i = 0; i < params->genomeLength; i ++ ){
        int j = irand( params->genomeLength-1 );
        while(taken[j])
            j = (j + 1) % params->genomeLength;
        taken[j] = true;
        bits[i] = j;
    }
}
void TspGenome::mutate(){
    if( roll(params->mutationProb)  ){
        int i = irand(params->genomeLength-1);
        assert( params->genomeLength > 0 );
        int j;
        do{ 
            j = irand(params->genomeLength-1);
        } while( i == j );
        swap( bits[i], bits[j] );
    }
    if( roll( ((TspParams*)params)->shiftProb ) ){
        int i = irand(params->genomeLength-1);
        int newSlot = irand( params->genomeLength-1 );
        int tmp = bits[i];
        int j;
        if (newSlot > i ){
            for( j = i; j < newSlot; j++ )
                bits[j] = bits[j+1];
            bits[j] = tmp;
        }
        else{
            for( j = i; j > newSlot; j-- )
                bits[j] = bits[j-1];
            bits[j] = tmp;
        }
    }
}

void TspGenome::crossoverWith( Genome* rhs ){
    static int* pos[2] = { new int[params->genomeLength], new int[params->genomeLength] };
    if( roll( params->crossoverProb ) ){
        for( int i = 0; i < params->genomeLength; i ++ ){
            pos[0][bits[i]] = i;
            pos[1][rhs->bits[i]] = i;
        }
        int a, b;
        do{
            a = irand( params->genomeLength-1 );
            b = irand( params->genomeLength-1 );
        } while( a == b );
        if( b < a )
            swap( a, b );
        assert( a < b );
        assert( a < params->genomeLength );
        assert( b < params->genomeLength );
        int i, j, m, n;
        for( i = a; i < b; i ++ ){
            m = bits[i];
            n = rhs->bits[i];

            swap( bits[i], bits[pos[0][n]] );
            swap( bits[i], bits[pos[1][m]] );
            }
    }
}
#define cost(a,b) graph->cost(a,b)
#define tour(b,c,d) cost(a,b)+cost(b,c)+cost(c,d)+cost(d,e)
#define swapb(a,b) swap(bits[a],bits[b]);
void TspGenome::memetic(){
//    static int swapped;
//    static int id = 0;
//    if( id%50 == 0 ){
//        printf("swapped: %i\n", swapped );
//        swapped = 0;
//    }
//    id++;
    int a = irand( params->genomeLength-1 );
    int b = ( a + 1 ) % params->genomeLength;
    int c = ( b + 1 ) % params->genomeLength;
    int d = ( c + 1 ) % params->genomeLength;
    int e = ( d + 1 ) % params->genomeLength;
    float cost[6];
    Graph* graph = ((TspParams*)params)->graph;
    cost[0] = tour( b,c,d );
    cost[1] = tour( b,d,c );
    cost[2] = tour( c,b,d );
    cost[3] = tour( c,d,b );
    cost[4] = tour( d,b,c );
    cost[5] = tour( d,c,b );
    float min = cost[0];
    int minc = 0;
    for( int i = 1; i < 5; i ++ )
        if( cost[i] < min ){
            min = cost[i];
            minc = i;
        }
    if( minc != 0 ){
        //printf("Savings: %f\n", cost[0] - min );
//    swapped++;
        switch( minc ){
            case 1: swapb( c,d ); break;
            case 2: swapb( b,c ); break;
            case 3: swapb( b,c );
                swapb( b,d ); break;
            case 4: swapb( d,c );
                    swapb( b,d ); break;
            case 5: swapb( b,d ); break;
        }
    }
//    assert( isvalid() );
}
nString TspGenome::toString(){
    char string[5024];
    string[0] = '\0';
    for( int i = 0; i < params->genomeLength; i ++ ){
        sprintf( string, "%s %2i", string, bits[i] );
    }
    char string2[1024];
    if( evaluated ){
        sprintf( string2, "f: %0.2f mf: %0.2f %s", fitness, mappedFitness, string );
        return nString(string2);
    }
    else
        return string;
}
