import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.ButtonGroup;
class OutErrReader implements Runnable {
    InputStream is;
    public OutErrReader( InputStream is ){
        this.is = is;
    }
    public void run(){
        try {
            BufferedReader in = new BufferedReader( new InputStreamReader ( is ) ) ;
            String temp = null;
            while(( temp = in.readLine()) != null ){
                System.out.println( temp );
            }
            is.close();
        }
        catch( Exception e) {
            e.printStackTrace();
        }
    }
}
public class Viewer extends JApplet implements ActionListener
{
    JFrame frame;
    Genomes genomes;
    
    public static void main( String[] args ) {
        new Viewer();
    }
    public void init(){
        System.out.println( "init()" );
    }
    public Viewer(){
        System.out.println( "Viewer()" );
        
        frame = new JFrame("Genome Viewer");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setSize( new Dimension( 700,700 ) );

        JPanel buttons = new JPanel();
            JButton update = new JButton("Update");
            update.addActionListener( this );
            buttons.add( update );
            JButton showSelected= new JButton("Show");
            showSelected.addActionListener( this );
            buttons.add( showSelected );

            JLabel view = new JLabel( "view: " );
            JRadioButton viewAll = new JRadioButton("All");
                viewAll.addActionListener( this );
                viewAll.setActionCommand("view all");
                buttons.add( view );

            JRadioButton viewBest = new JRadioButton("Best");
                viewBest.addActionListener( this );
                viewBest.setActionCommand("view best");
                buttons.add( viewAll );

            JRadioButton viewImprovements = new JRadioButton("Improvments");
                viewImprovements.addActionListener( this );
                viewImprovements.setActionCommand("view improvements");
                buttons.add( viewImprovements );

            ButtonGroup viewGroup = new ButtonGroup();
                viewGroup.add( viewAll );
                viewGroup.add( viewBest );
                viewGroup.add( viewImprovements );

            buttons.add( viewBest );
            
        frame.getContentPane().add( buttons, BorderLayout.NORTH );
        genomes = new Genomes();
        frame.getContentPane().add( genomes, BorderLayout.CENTER );

        frame.setVisible( true );
        update( new File("../../logs/genlogs") );
    }
    public void actionPerformed( ActionEvent e ){
        String s = e.getActionCommand();
        System.out.println( "do: " + s );
        if( s.equals("Update") )
            update( new File( "../../logs/genlogs") );
        else if( s.equals("Show"))
            showSelected();
        else if( s.equals("view all" ) )
            viewAll();
        else if( s.equals("view best" ) )
            viewBest();
        else if( s.equals("view improvements" ) )
            viewImprovements();
    }
    public void update( File file ){
        BufferedReader fin = null;
        try {   
            fin = new BufferedReader( new FileReader( file ) ); 
        } catch ( java.io.IOException e ) { 
            e.printStackTrace(); 
        }
        String s;
        int generation = -1;
        float fitmin, fitmax, fitavg;
        float mfitmin, mfitmax, mfitavg;
        float fit, mfit;
        String bits;
        try {
            while( true ){
                s = fin.readLine();
                if( s == null ) break;
                //System.out.println( "read string : " + s );
                String sec[] = s.split("[: \t]+");
                //System.out.println( "sec[0] : *" + sec[0] + "*");

                if( sec[0].equals("generation") ){ 
                    generation = c.toInt( sec[1] );
                    //System.out.println( "generation " + generation );
                }
                else if( sec[0].equals("fitnessstats") ){
                    fitmin = c.toFloat( sec[2] );
                    fitmax = c.toFloat( sec[4] );
                    fitavg = c.toFloat( sec[6] );
                    //System.out.println( "stats : " + fitmin + " " + fitmax + " " + fitavg );
                }
                else if( sec[0].equals("mappedfitnessstats") ){
                    mfitmin = c.toFloat( sec[2] );
                    mfitmax = c.toFloat( sec[4] );
                    mfitavg = c.toFloat( sec[6] );
                    //System.out.println( "mstats : " + mfitmin + " " + mfitmax + " " + mfitavg );
                }
                else if( sec[0].equals("f") ){
                    fit = c.toFloat(sec[1]);
                    mfit = c.toFloat( sec[3] );
                    bits = sec[4];
                    //System.out.println( "genome $" + fit + "($" + mfit + ") " + bits );
                    genomes.add( generation, fit, mfit, bits );
                }
            }
            //System.out.println( "done parsing" );
            fin.close();
        }
        catch( java.io.IOException e ){
            e.printStackTrace();
        }
    }
    public void showSelected( ){
        try {
            String s = genomes.getSelected();

            String command = "../../go show " + s;
            //String[] args = {"show", s};
            Process pro = null;
            //pro = Runtime.getRuntime().exec( command, args );
            pro = Runtime.getRuntime().exec( command );
            InputStream error = pro.getErrorStream();
            InputStream output = pro.getInputStream();
            Thread err = new Thread( new OutErrReader( error ) );
            Thread out = new Thread( new OutErrReader( output ) );
            out.start();
            err.start();
            pro.waitFor();
        }
        catch ( java.io.IOException e ){
            e.printStackTrace(); }
        catch ( java.lang.InterruptedException e ){
            e.printStackTrace();
        }
    }
    public void viewAll(){
        genomes.viewAll();
    }
    public void viewBest(){
        genomes.viewBest();
    }
    public void viewImprovements(){
        genomes.viewImprovements();
    }
}
