#include "genome.h"
Genome::Genome(){ 
    params = 0;
    bits = 0;
    evaluated = false;
    lastSlot = 0;
}
Genome::~Genome(){
    if( bits )
        delete[] bits;
}

void Genome::setParams( Params* params ){
    this->params = params;
}
void Genome::initialize(){
    bits = new int[params->genomeLength];
    for( int i = 0; i < params->genomeLength; i ++ )
        bits[i] = 0;
}
nString Genome::toString(){
    char string[1024];
    string[0] = '\0';
    for( int i = 0; i < params->genomeLength; i ++ ){
        sprintf( string, "%s%i", string, bits[i] );
    }
    char string2[1024];
    if( evaluated ){
        sprintf( string2, "f: %6.2f mf: %6.2f %s", fitness, mappedFitness, string );
        return nString(string2);
    }
    else
        return string;
}
void Genome::operator = ( const Genome& rhs ){
    params = rhs.params;
    //assert( bits );
    if( bits==0 )
        bits = new int[params->genomeLength];
    //    delete[] bits;
    memcpy( bits, rhs.bits, sizeof( bits[0] ) * params->genomeLength );
    //for( int i = 0; i < params->genomeLength; i ++ )
    //    assert( bits[i] == rhs.bits[i] );
    //assert( isvalid() );
    fitness = rhs.fitness;
    mappedFitness = rhs.mappedFitness;
    evaluated = rhs.evaluated;
}
void Genome::eval(){
    float val;
    if( params->bitTree ){
        if(params->bitTree->find( bits, val ) ) {
            fitness = val;
            //printf("prior: %s\n", toString().Get() );
        }
        else {
            fitness = params->evaluator->evaluate( this );
            params->bitTree->insert( this->bits, fitness );
        }
    }
    else
        fitness = params->evaluator->evaluate( this );
    evaluated = true;
}
void Genome::randomize(){
    for( int i = 0; i < params->genomeLength; i ++ )
        bits[i] = rand()%2;
    evaluated = false;
}
void Genome::mutate(){
    for( int i = 0; i < params->genomeLength; i ++ )
        if( drand48() <=  params->mutationProb ){
            bits[i] = 1 - bits[i];
            evaluated = false;
        }
}
void Genome::crossoverWith( Genome* rhs ){
    if( drand48() > params->crossoverProb ) return;
    assert( params->genomeLength != 0 );
    int a = rand()%params->genomeLength;
    int b = rand()%params->genomeLength;
    for( int i = a; i != b; i = (i+1)%params->genomeLength ){
        int t = bits[i];
        bits[i] = rhs->bits[i];
        rhs->bits[i] = t;
    }
    evaluated = false;
}

void Genome::fromString( const char* s ){
    printf("bits: ");
    for( int i = 0; i < strlen( s ); i ++ ){
        bits[i] = s[i] - '0';
        printf("%i", bits[i] );
    }
    printf("\n");
}

void Genome::memetic(){
}
